@@ -39,16 +39,22 @@ type Runner struct {
 	queue *queue.Queue
 }
 
-// buildCanonicalConfigPath computes the canonical config path for a unit.
-// It handles .hcl/.json suffixes, joins DefaultTerragruntConfigPath when needed,
+// BuildCanonicalConfigPath computes the canonical config path for a unit.
+// It handles .hcl/.json suffixes, joins an appropriate config filename when needed,
 // converts to canonical absolute path, and updates the unit's path.
 // Returns the canonical config path and the canonical unit directory.
-func buildCanonicalConfigPath(unit *component.Unit, basePath string) (canonicalConfigPath string, canonicalDir string, err error) {
-	// Discovery paths are directories (e.g., "./other") not files
-	// We need to append the config filename to get the full config path
+func BuildCanonicalConfigPath(
+	unit *component.Unit,
+	basePath string,
+) (canonicalConfigPath string, canonicalDir string, err error) {
 	terragruntConfigPath := unit.Path()
 	if !strings.HasSuffix(terragruntConfigPath, ".hcl") && !strings.HasSuffix(terragruntConfigPath, ".json") {
-		terragruntConfigPath = filepath.Join(unit.Path(), config.DefaultTerragruntConfigPath)
+		fileName := config.DefaultTerragruntConfigPath
+		if unit.ConfigFile() != "" {
+			fileName = unit.ConfigFile()
+		}
+
+		terragruntConfigPath = filepath.Join(unit.Path(), fileName)
 	}
 
 	// Convert to canonical absolute path - this is critical for dependency resolution
@@ -67,10 +73,10 @@ func buildCanonicalConfigPath(unit *component.Unit, basePath string) (canonicalC
 	return canonicalConfigPath, canonicalDir, nil
 }
 
-// cloneUnitOptions clones TerragruntOptions for a specific unit.
+// CloneUnitOptions clones TerragruntOptions for a specific unit.
 // It handles CloneWithConfigPath, per-unit DownloadDir fallback, and OriginalTerragruntConfigPath.
 // Returns the cloned options and logger, or the original logger if stack has no options.
-func cloneUnitOptions(
+func CloneUnitOptions(
 	stack *component.Stack,
 	unit *component.Unit,
 	canonicalConfigPath string,
@@ -109,10 +115,10 @@ func cloneUnitOptions(
 	return clonedOpts, clonedLogger, nil
 }
 
-// shouldSkipUnitWithoutTerraform checks if a unit should be skipped because it has
+// ShouldSkipUnitWithoutTerraform checks if a unit should be skipped because it has
 // neither a Terraform source nor any Terraform/OpenTofu files in its directory.
 // Returns true if the unit should be skipped, false otherwise.
-func shouldSkipUnitWithoutTerraform(unit *component.Unit, dir string, l log.Logger) (bool, error) {
+func ShouldSkipUnitWithoutTerraform(unit *component.Unit, dir string, l log.Logger) (bool, error) {
 	terragruntConfig := unit.Config()
 
 	// If the unit has a Terraform source configured, don't skip it
@@ -160,13 +166,13 @@ func resolveUnitsFromDiscovery(
 		}
 
 		// Build canonical config path and update unit path
-		canonicalConfigPath, canonicalDir, err := buildCanonicalConfigPath(unit, basePath)
+		canonicalConfigPath, canonicalDir, err := BuildCanonicalConfigPath(unit, basePath)
 		if err != nil {
 			return nil, err
 		}
 
 		// Clone options for this unit
-		unitOpts, unitLogger, err := cloneUnitOptions(stack, unit, canonicalConfigPath, stackDefaultDownloadDir, l)
+		unitOpts, unitLogger, err := CloneUnitOptions(stack, unit, canonicalConfigPath, stackDefaultDownloadDir, l)
 		if err != nil {
 			return nil, err
 		}
@@ -193,7 +199,7 @@ func resolveUnitsFromDiscovery(
 		}
 
 		// Skip units without Terraform configuration
-		skip, err := shouldSkipUnitWithoutTerraform(unit, canonicalDir, unitLogger)
+		skip, err := ShouldSkipUnitWithoutTerraform(unit, canonicalDir, unitLogger)
 		if err != nil {
 			return nil, err
 		}
@@ -312,37 +318,6 @@ func NewRunnerPoolStack(
 
 	// Check for units with Git refs but no remote state configuration
 	checkLocalStateWithGitRefs(l, units)
-
-	// Record exclude-dir reasons in report before filtering.
-	if runner.Stack.Execution != nil && runner.Stack.Execution.Report != nil && len(terragruntOptions.ExcludeDirs) > 0 {
-		for _, unit := range units {
-			for _, dir := range terragruntOptions.ExcludeDirs {
-				cleanDir := dir
-				if !filepath.IsAbs(cleanDir) {
-					cleanDir = filepath.Join(terragruntOptions.WorkingDir, cleanDir)
-				}
-
-				cleanDir = util.CleanPath(cleanDir)
-
-				if util.HasPathPrefix(unit.Path(), cleanDir) {
-					absPath := util.CleanPath(unit.Path())
-					if !filepath.IsAbs(absPath) {
-						if abs, err := filepath.Abs(absPath); err == nil {
-							absPath = util.CleanPath(abs)
-						}
-					}
-
-					run, err := runner.Stack.Execution.Report.EnsureRun(absPath)
-					if err != nil {
-						continue
-					}
-
-					_ = runner.Stack.Execution.Report.EndRun(run.Path, report.WithResult(report.ResultExcluded), report.WithReason(report.ReasonExcludeDir))
-				}
-			}
-		}
-	}
-
 	runner.Stack.Units = units
 
 	if isDestroyCommand(terragruntOptions.TerraformCommand, terragruntOptions.TerraformCliArgs) {
@@ -459,7 +434,7 @@ func (r *Runner) Run(ctx context.Context, l log.Logger, opts *options.Terragrunt
 		}
 	}
 
-	if util.ListContainsElement(config.TerraformCommandsNeedInput, terraformCmd) {
+	if slices.Contains(config.TerraformCommandsNeedInput, terraformCmd) {
 		opts.TerraformCliArgs = util.StringListInsert(opts.TerraformCliArgs, "-input=false", 1)
 		r.syncTerraformCliArgs(l, opts)
 	}
@@ -489,7 +464,7 @@ func (r *Runner) Run(ctx context.Context, l log.Logger, opts *options.Terragrunt
 				// Ensure path is absolute for reporting
 				unitPath := u.AbsolutePath()
 
-				run, err := r.Stack.Execution.Report.EnsureRun(unitPath)
+				run, err := r.Stack.Execution.Report.EnsureRun(l, unitPath)
 				if err != nil {
 					l.Errorf("Error ensuring run for unit %s: %v", unitPath, err)
 					continue
@@ -507,6 +482,7 @@ func (r *Runner) Run(ctx context.Context, l log.Logger, opts *options.Terragrunt
 					}
 
 					if err := r.Stack.Execution.Report.EndRun(
+						l,
 						run.Path,
 						report.WithResult(report.ResultExcluded),
 						report.WithReason(reason),
@@ -577,7 +553,7 @@ func (r *Runner) Run(ctx context.Context, l log.Logger, opts *options.Terragrunt
 				// Ensure path is absolute for reporting
 				unitPath := unit.AbsolutePath()
 
-				run, reportErr := r.Stack.Execution.Report.EnsureRun(unitPath)
+				run, reportErr := r.Stack.Execution.Report.EnsureRun(l, unitPath)
 				if reportErr != nil {
 					l.Errorf("Error ensuring run for early exit unit %s: %v", unitPath, reportErr)
 					continue
@@ -608,7 +584,7 @@ func (r *Runner) Run(ctx context.Context, l log.Logger, opts *options.Terragrunt
 					endOpts = append(endOpts, report.WithCauseAncestorExit(failedAncestor))
 				}
 
-				if endErr := r.Stack.Execution.Report.EndRun(run.Path, endOpts...); endErr != nil {
+				if endErr := r.Stack.Execution.Report.EndRun(l, run.Path, endOpts...); endErr != nil {
 					l.Errorf("Error ending run for early exit unit %s: %v", unitPath, endErr)
 				}
 			}
@@ -922,22 +898,22 @@ func FilterDiscoveredUnits(discovered component.Components, units []*component.U
 
 		// Add any missing allowed dependencies from the resolved unit graph
 		for _, dep := range u.Dependencies() {
-			depUnit, ok := dep.(*component.Unit)
-			if !ok || depUnit == nil {
+			depUnit, okDep := dep.(*component.Unit)
+			if !okDep || depUnit == nil {
 				continue
 			}
 
-			if _, ok := allowed[depUnit.Path()]; !ok {
+			if _, allowedOK := allowed[depUnit.Path()]; !allowedOK {
 				continue
 			}
 
-			if _, ok := existing[depUnit.Path()]; ok {
+			if _, existsOK := existing[depUnit.Path()]; existsOK {
 				continue
 			}
 
 			// Ensure the dependency config exists in the filtered set
-			depCfg, ok := present[depUnit.Path()]
-			if !ok {
+			depCfg, presentOK := present[depUnit.Path()]
+			if !presentOK {
 				depCfg = component.NewUnit(depUnit.Path())
 				filtered = append(filtered, depCfg)
 				present[depUnit.Path()] = depCfg
